@@ -15,7 +15,7 @@ typedef Eo Evas_Canvas3D_Node;
 #endif
 #define EVAS_CANVAS3D_NODE_CLASS evas_canvas3d_node_class_get()
 
-EAPI const Eo_Class *evas_canvas3d_node_class_get(void) EINA_CONST;
+EWAPI const Eo_Class *evas_canvas3d_node_class_get(void);
 
 /**
  * @brief Set the position inheritance flag of the given node.
@@ -28,7 +28,7 @@ EAPI const Eo_Class *evas_canvas3d_node_class_get(void) EINA_CONST;
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_position_inherit_set(Eina_Bool inherit);
+EOAPI void evas_canvas3d_node_position_inherit_set(Eo *obj, Eina_Bool inherit);
 
 /**
  * @brief Get the position inheritance flag of the given node.
@@ -39,7 +39,7 @@ EOAPI void evas_canvas3d_node_position_inherit_set(Eina_Bool inherit);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Eina_Bool evas_canvas3d_node_position_inherit_get(void);
+EOAPI Eina_Bool evas_canvas3d_node_position_inherit_get(const Eo *obj);
 
 /**
  * @brief Set the orientation inheritance flag of the given node.
@@ -53,7 +53,7 @@ EOAPI Eina_Bool evas_canvas3d_node_position_inherit_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_orientation_inherit_set(Eina_Bool inherit);
+EOAPI void evas_canvas3d_node_orientation_inherit_set(Eo *obj, Eina_Bool inherit);
 
 /**
  * @brief Get the orientation inheritance flag of the given node.
@@ -64,7 +64,7 @@ EOAPI void evas_canvas3d_node_orientation_inherit_set(Eina_Bool inherit);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Eina_Bool evas_canvas3d_node_orientation_inherit_get(void);
+EOAPI Eina_Bool evas_canvas3d_node_orientation_inherit_get(const Eo *obj);
 
 /**
  * @brief Set the scale inheritance flag of the given node.
@@ -77,7 +77,7 @@ EOAPI Eina_Bool evas_canvas3d_node_orientation_inherit_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_scale_inherit_set(Eina_Bool inherit);
+EOAPI void evas_canvas3d_node_scale_inherit_set(Eo *obj, Eina_Bool inherit);
 
 /**
  * @brief Get the scale inheritance flag of the given node.
@@ -88,7 +88,7 @@ EOAPI void evas_canvas3d_node_scale_inherit_set(Eina_Bool inherit);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Eina_Bool evas_canvas3d_node_scale_inherit_get(void);
+EOAPI Eina_Bool evas_canvas3d_node_scale_inherit_get(const Eo *obj);
 
 /**
  * @brief Set a camera to the given node.
@@ -103,7 +103,7 @@ EOAPI Eina_Bool evas_canvas3d_node_scale_inherit_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_camera_set(Evas_Canvas3D_Object *camera);
+EOAPI void evas_canvas3d_node_camera_set(Eo *obj, Evas_Canvas3D_Object *camera);
 
 /**
  * @brief Get the camera of the given node.
@@ -114,7 +114,7 @@ EOAPI void evas_canvas3d_node_camera_set(Evas_Canvas3D_Object *camera);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Evas_Canvas3D_Object *evas_canvas3d_node_camera_get(void);
+EOAPI Evas_Canvas3D_Object *evas_canvas3d_node_camera_get(const Eo *obj);
 
 /**
  * @brief Set the light of the given node.
@@ -129,7 +129,7 @@ EOAPI Evas_Canvas3D_Object *evas_canvas3d_node_camera_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_light_set(Evas_Canvas3D_Light *light);
+EOAPI void evas_canvas3d_node_light_set(Eo *obj, Evas_Canvas3D_Light *light);
 
 /**
  * @brief Get the light of the given node.
@@ -140,7 +140,7 @@ EOAPI void evas_canvas3d_node_light_set(Evas_Canvas3D_Light *light);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Evas_Canvas3D_Light *evas_canvas3d_node_light_get(void);
+EOAPI Evas_Canvas3D_Light *evas_canvas3d_node_light_get(const Eo *obj);
 
 /**
  * @brief Set the animation frame number of the given node for the given mesh.
@@ -158,7 +158,7 @@ EOAPI Evas_Canvas3D_Light *evas_canvas3d_node_light_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_mesh_frame_set(Evas_Canvas3D_Mesh * mesh, int frame);
+EOAPI void evas_canvas3d_node_mesh_frame_set(Eo *obj, Evas_Canvas3D_Mesh * mesh, int frame);
 
 /**
  * @brief Set the animation frame number of the given node for the given mesh.
@@ -175,7 +175,7 @@ EOAPI void evas_canvas3d_node_mesh_frame_set(Evas_Canvas3D_Mesh * mesh, int fram
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI int evas_canvas3d_node_mesh_frame_get(Evas_Canvas3D_Mesh * mesh);
+EOAPI int evas_canvas3d_node_mesh_frame_get(const Eo *obj, Evas_Canvas3D_Mesh * mesh);
 
 /**
  * @brief Set behavior of node like billboard object.
@@ -185,7 +185,7 @@ EOAPI int evas_canvas3d_node_mesh_frame_get(Evas_Canvas3D_Mesh * mesh);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_billboard_target_set(Evas_Canvas3D_Node *target);
+EOAPI void evas_canvas3d_node_billboard_target_set(Eo *obj, Evas_Canvas3D_Node *target);
 
 /**
  * @brief Get the target node for billboard object.
@@ -197,7 +197,27 @@ EOAPI void evas_canvas3d_node_billboard_target_set(Evas_Canvas3D_Node *target);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Evas_Canvas3D_Node *evas_canvas3d_node_billboard_target_get(void);
+EOAPI Evas_Canvas3D_Node *evas_canvas3d_node_billboard_target_get(const Eo *obj);
+
+/**
+ * @brief Enable behavior of node like LOD object.
+ *
+ * @param[in] enable status property (true/false)
+ *
+ * @ingroup Evas_Canvas3D_Node
+ */
+EOAPI void evas_canvas3d_node_lod_enable_set(Eo *obj, Eina_Bool enable);
+
+/**
+ * @brief Get(check) status of node does node is LOD object.
+ *
+ * See also @ref evas_canvas3d_node_lod_enable_set.
+ *
+ * @return status property (true/false)
+ *
+ * @ingroup Evas_Canvas3D_Node
+ */
+EOAPI Eina_Bool evas_canvas3d_node_lod_enable_get(const Eo *obj);
 
 /**
  * @brief Constructor.
@@ -206,7 +226,7 @@ EOAPI Evas_Canvas3D_Node *evas_canvas3d_node_billboard_target_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_constructor(Evas_Canvas3D_Node_Type type);
+EOAPI void evas_canvas3d_node_constructor(Eo *obj, Evas_Canvas3D_Node_Type type);
 
 /**
  * @brief Get the type of the given node.
@@ -217,7 +237,7 @@ EOAPI void evas_canvas3d_node_constructor(Evas_Canvas3D_Node_Type type);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Evas_Canvas3D_Node_Type evas_canvas3d_node_type_get(void);
+EOAPI Evas_Canvas3D_Node_Type evas_canvas3d_node_type_get(const Eo *obj);
 
 /**
  * @brief Add a member node to the given node.
@@ -231,7 +251,7 @@ EOAPI Evas_Canvas3D_Node_Type evas_canvas3d_node_type_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_member_add(Evas_Canvas3D_Node *member);
+EOAPI void evas_canvas3d_node_member_add(Eo *obj, Evas_Canvas3D_Node *member);
 
 /**
  * @brief Delete a member node from the given node.
@@ -242,7 +262,7 @@ EOAPI void evas_canvas3d_node_member_add(Evas_Canvas3D_Node *member);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_member_del(Evas_Canvas3D_Node *member);
+EOAPI void evas_canvas3d_node_member_del(Eo *obj, Evas_Canvas3D_Node *member);
 
 /**
  * @brief Get the parent node of the given node.
@@ -253,7 +273,7 @@ EOAPI void evas_canvas3d_node_member_del(Evas_Canvas3D_Node *member);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Evas_Canvas3D_Node *evas_canvas3d_node_parent_get(void);
+EOAPI Evas_Canvas3D_Node *evas_canvas3d_node_parent_get(const Eo *obj);
 
 /**
  * @brief Get the list of member nodes of the given node.
@@ -264,16 +284,16 @@ EOAPI Evas_Canvas3D_Node *evas_canvas3d_node_parent_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI const Eina_List *evas_canvas3d_node_member_list_get(void);
+EOAPI const Eina_List *evas_canvas3d_node_member_list_get(const Eo *obj);
 
 /**
  * @brief Get hash table of scenes using this node as root.
  *
- * @return scenes using this node as root.
+ * @return scenes using this node as root. FIXME: Actually a scene, not object.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI Eina_Hash *evas_canvas3d_node_scene_root_get(void);
+EOAPI Eina_Hash *evas_canvas3d_node_scene_root_get(Eo *obj);
 
 /**
  * @brief Set the position of the given node.
@@ -285,12 +305,13 @@ EOAPI Eina_Hash *evas_canvas3d_node_scene_root_get(void);
  *
  * See also @ref evas_canvas3d_node_position_inherit_set.
  *
+ * @param[in] x X coordinate of the position.
  * @param[in] y Y coordinate of the position.
  * @param[in] z Z coordinate of the position.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_position_set(Evas_Real x, Evas_Real y, Evas_Real z);
+EOAPI void evas_canvas3d_node_position_set(Eo *obj, Evas_Real x, Evas_Real y, Evas_Real z);
 
 /**
  * @brief Set the orientation of the given node using quaternion.
@@ -302,26 +323,28 @@ EOAPI void evas_canvas3d_node_position_set(Evas_Real x, Evas_Real y, Evas_Real z
  *
  * See also @ref evas_canvas3d_node_orientation_inherit_set.
  *
+ * @param[in] x X term of the orientation quaternion (w, x, y, z.
  * @param[in] y Y term of the orientation quaternion (w, x, y, z.
  * @param[in] z Z term of the orientation quaternion (w, x, y, z.
  * @param[in] w W term of the orientation quaternion (w, x, y, z.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_orientation_set(Evas_Real x, Evas_Real y, Evas_Real z, Evas_Real w);
+EOAPI void evas_canvas3d_node_orientation_set(Eo *obj, Evas_Real x, Evas_Real y, Evas_Real z, Evas_Real w);
 
 /**
  * @brief Set the orientation of the given node using axis-angle.
  *
  * See also @ref evas_canvas3d_node_orientation_set.
  *
+ * @param[in] angle Rotation angle.
  * @param[in] x X term of the rotation axis.
  * @param[in] y Y term of the rotation axis.
  * @param[in] z Z term of the rotation axis.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_orientation_angle_axis_set(Evas_Real angle, Evas_Real x, Evas_Real y, Evas_Real z);
+EOAPI void evas_canvas3d_node_orientation_angle_axis_set(Eo *obj, Evas_Real angle, Evas_Real x, Evas_Real y, Evas_Real z);
 
 /**
  * @brief Set the scale of the given node.
@@ -334,31 +357,34 @@ EOAPI void evas_canvas3d_node_orientation_angle_axis_set(Evas_Real angle, Evas_R
  *
  * See also @ref evas_canvas3d_node_scale_inherit_set.
  *
+ * @param[in] x Scale factor along X-axis.
  * @param[in] y Scale factor along Y-axis.
  * @param[in] z Scale factor along Z-axis.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_scale_set(Evas_Real x, Evas_Real y, Evas_Real z);
+EOAPI void evas_canvas3d_node_scale_set(Eo *obj, Evas_Real x, Evas_Real y, Evas_Real z);
 
 /**
  * @brief Get the position of the given node.
  *
  * See also @ref evas_canvas3d_node_position_set.
  *
+ * @param[in] space The given node.
  * @param[out] x Pointer to receive X coordinate of the position.
  * @param[out] y Pointer to receive Y coordinate of the position.
  * @param[out] z Pointer to receive Z coordinate of the position.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_position_get(Evas_Canvas3D_Space space, Evas_Real *x, Evas_Real *y, Evas_Real *z);
+EOAPI void evas_canvas3d_node_position_get(const Eo *obj, Evas_Canvas3D_Space space, Evas_Real *x, Evas_Real *y, Evas_Real *z);
 
 /**
  * @brief Get the orientation of the given node as quaternion.
  *
  * See also @ref evas_canvas3d_node_orientation_set.
  *
+ * @param[in] space The given node.
  * @param[out] x Pointer to receive X term of the orientation quaternion.
  * @param[out] y Pointer to receive Y term of the orientation quaternion.
  * @param[out] z Pointer to receive Z term of the orientation quaternion.
@@ -366,20 +392,21 @@ EOAPI void evas_canvas3d_node_position_get(Evas_Canvas3D_Space space, Evas_Real
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_orientation_get(Evas_Canvas3D_Space space, Evas_Real *x, Evas_Real *y, Evas_Real *z, Evas_Real *w);
+EOAPI void evas_canvas3d_node_orientation_get(const Eo *obj, Evas_Canvas3D_Space space, Evas_Real *x, Evas_Real *y, Evas_Real *z, Evas_Real *w);
 
 /**
  * @brief Get the scale of the given node.
  *
  * See also @ref evas_canvas3d_node_scale_get.
  *
+ * @param[in] space The given node.
  * @param[out] x Pointer to receive Scale factor along X-axis.
  * @param[out] y Pointer to receive Scale factor along X-axis.
  * @param[out] z Pointer to receive Scale factor along X-axis.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_scale_get(Evas_Canvas3D_Space space, Evas_Real *x, Evas_Real *y, Evas_Real *z);
+EOAPI void evas_canvas3d_node_scale_get(const Eo *obj, Evas_Canvas3D_Space space, Evas_Real *x, Evas_Real *y, Evas_Real *z);
 
 /**
  * @brief Rotate the given node to look at desired position.
@@ -390,6 +417,7 @@ EOAPI void evas_canvas3d_node_scale_get(Evas_Canvas3D_Space space, Evas_Real *x,
  *
  * See also @ref evas_canvas3d_node_orientation_set.
  *
+ * @param[in] target_space Space where the target position belongs to.
  * @param[in] x X coordinate of the target position.
  * @param[in] y Y coordinate of the target position.
  * @param[in] z Z coordinate of the target position.
@@ -400,7 +428,7 @@ EOAPI void evas_canvas3d_node_scale_get(Evas_Canvas3D_Space space, Evas_Real *x,
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_look_at_set(Evas_Canvas3D_Space target_space, Evas_Real x, Evas_Real y, Evas_Real z, Evas_Canvas3D_Space up_space, Evas_Real ux, Evas_Real uy, Evas_Real uz);
+EOAPI void evas_canvas3d_node_look_at_set(Eo *obj, Evas_Canvas3D_Space target_space, Evas_Real x, Evas_Real y, Evas_Real z, Evas_Canvas3D_Space up_space, Evas_Real ux, Evas_Real uy, Evas_Real uz);
 
 /**
  * @brief Add a mesh to the given node.
@@ -414,7 +442,7 @@ EOAPI void evas_canvas3d_node_look_at_set(Evas_Canvas3D_Space target_space, Evas
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_mesh_add(Evas_Canvas3D_Mesh *mesh);
+EOAPI void evas_canvas3d_node_mesh_add(Eo *obj, Evas_Canvas3D_Mesh *mesh);
 
 /**
  * @brief Delete a mesh from the given node.
@@ -429,7 +457,7 @@ EOAPI void evas_canvas3d_node_mesh_add(Evas_Canvas3D_Mesh *mesh);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_mesh_del(Evas_Canvas3D_Mesh *mesh);
+EOAPI void evas_canvas3d_node_mesh_del(Eo *obj, Evas_Canvas3D_Mesh *mesh);
 
 /**
  * @brief Get the list of meshes of the given node.
@@ -444,11 +472,12 @@ EOAPI void evas_canvas3d_node_mesh_del(Evas_Canvas3D_Mesh *mesh);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI const Eina_List *evas_canvas3d_node_mesh_list_get(void);
+EOAPI const Eina_List *evas_canvas3d_node_mesh_list_get(const Eo *obj);
 
 /**
  * @brief Get axis-aligned bounding box (AABB) of the given node.
  *
+ * @param[in] x Pointer to receive X coordinate of the first point of AABB.
  * @param[in] y Pointer to receive Y coordinate of the first point of AABB.
  * @param[in] z Pointer to receive Z coordinate of the first point of AABB.
  * @param[in] x2 Pointer to receive X coordinate of the second point of AABB.
@@ -457,18 +486,19 @@ EOAPI const Eina_List *evas_canvas3d_node_mesh_list_get(void);
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_bounding_box_get(Evas_Real *x, Evas_Real *y, Evas_Real *z, Evas_Real *x2, Evas_Real *y2, Evas_Real *z2);
+EOAPI void evas_canvas3d_node_bounding_box_get(Eo *obj, Evas_Real *x, Evas_Real *y, Evas_Real *z, Evas_Real *x2, Evas_Real *y2, Evas_Real *z2);
 
 /**
  * @brief Get bounding sphere of the given node.
  *
+ * @param[in] x Pointer to receive X coordinate of the center of sphere.
  * @param[in] y Pointer to receive Y coordinate of the center of sphere.
  * @param[in] z Pointer to receive Z coordinate of center of sphere.
  * @param[in] r Pointer to receive radius of center of sphere.
  *
  * @ingroup Evas_Canvas3D_Node
  */
-EOAPI void evas_canvas3d_node_bounding_sphere_get(Evas_Real *x, Evas_Real *y, Evas_Real *z, Evas_Real *r);
+EOAPI void evas_canvas3d_node_bounding_sphere_get(Eo *obj, Evas_Real *x, Evas_Real *y, Evas_Real *z, Evas_Real *r);
 
 
 #endif
