@@ -15,7 +15,7 @@ typedef Eo Evas_Canvas3D_Scene;
 #endif
 #define EVAS_CANVAS3D_SCENE_CLASS evas_canvas3d_scene_class_get()
 
-EAPI const Eo_Class *evas_canvas3d_scene_class_get(void) EINA_CONST;
+EWAPI const Eo_Class *evas_canvas3d_scene_class_get(void);
 
 /**
  * @brief Set the root node of a scene.
@@ -24,7 +24,7 @@ EAPI const Eo_Class *evas_canvas3d_scene_class_get(void) EINA_CONST;
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_root_node_set(Evas_Canvas3D_Node *node);
+EOAPI void evas_canvas3d_scene_root_node_set(Eo *obj, Evas_Canvas3D_Node *node);
 
 /**
  * @brief Get the root node of a scene.
@@ -33,7 +33,7 @@ EOAPI void evas_canvas3d_scene_root_node_set(Evas_Canvas3D_Node *node);
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_root_node_get(void);
+EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_root_node_get(const Eo *obj);
 
 /**
  * @brief Set the camera node of a scene.
@@ -42,7 +42,7 @@ EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_root_node_get(void);
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_camera_node_set(Evas_Canvas3D_Node *node);
+EOAPI void evas_canvas3d_scene_camera_node_set(Eo *obj, Evas_Canvas3D_Node *node);
 
 /**
  * @brief Get the camera node of a scene.
@@ -51,7 +51,7 @@ EOAPI void evas_canvas3d_scene_camera_node_set(Evas_Canvas3D_Node *node);
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_camera_node_get(void);
+EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_camera_node_get(const Eo *obj);
 
 /**
  * @brief Set scale and units used to calculate depth values.
@@ -62,7 +62,7 @@ EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_camera_node_get(void);
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_shadows_depth_set(Evas_Real depth_offset, Evas_Real depth_constant);
+EOAPI void evas_canvas3d_scene_shadows_depth_set(Eo *obj, Evas_Real depth_offset, Evas_Real depth_constant);
 
 /**
  * @brief Get scale and units used to calculate depth values.
@@ -73,7 +73,7 @@ EOAPI void evas_canvas3d_scene_shadows_depth_set(Evas_Real depth_offset, Evas_Re
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_shadows_depth_get(Evas_Real *depth_offset, Evas_Real *depth_constant);
+EOAPI void evas_canvas3d_scene_shadows_depth_get(const Eo *obj, Evas_Real *depth_offset, Evas_Real *depth_constant);
 
 /**
  * @brief Set the resolution of a scene.
@@ -81,20 +81,22 @@ EOAPI void evas_canvas3d_scene_shadows_depth_get(Evas_Real *depth_offset, Evas_R
  * A scene should be rendered to be displayed through an image objects. The
  * resolution defines size of the internal surface holding the rendered result.
  *
+ * @param[in] w Width of the resolution.
  * @param[in] h Height of the resolution.
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_size_set(int w, int h);
+EOAPI void evas_canvas3d_scene_size_set(Eo *obj, int w, int h);
 
 /**
  * @brief Get the internal resolution of a scene.
  *
+ * @param[out] w Pointer to receive width of the resolution.
  * @param[out] h Pointer to receive height of the resolution.
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_size_get(int *w, int *h);
+EOAPI void evas_canvas3d_scene_size_get(const Eo *obj, int *w, int *h);
 
 /**
  * @brief Set the background color of a scene.
@@ -105,24 +107,26 @@ EOAPI void evas_canvas3d_scene_size_get(int *w, int *h);
  *
  * Default background color is (0.0, 0.0, 0.0, 0.0).
  *
+ * @param[in] r Red component of the background color.
  * @param[in] g Green component of the background color.
  * @param[in] b Blue component of the background color.
  * @param[in] a Alpha component of the background color.
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_background_color_set(Evas_Real r, Evas_Real g, Evas_Real b, Evas_Real a);
+EOAPI void evas_canvas3d_scene_background_color_set(Eo *obj, Evas_Real r, Evas_Real g, Evas_Real b, Evas_Real a);
 
 /**
  * @brief Get the background color of a scene.
  *
+ * @param[out] r Pointer to receive red component of the background color.
  * @param[out] g Pointer to receive green component of the background color.
  * @param[out] b Pointer to receive blue component of the background color.
  * @param[out] a Pointer to receive alpha component of the background color.
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_background_color_get(Evas_Real *r, Evas_Real *g, Evas_Real *b, Evas_Real *a);
+EOAPI void evas_canvas3d_scene_background_color_get(const Eo *obj, Evas_Real *r, Evas_Real *g, Evas_Real *b, Evas_Real *a);
 
 /**
  * @brief Get information on the most front visible mesh for the given
@@ -132,6 +136,7 @@ EOAPI void evas_canvas3d_scene_background_color_get(Evas_Real *r, Evas_Real *g,
  * 0) and right-bottom is (w, h) where (w, h) is the size of the scene. The
  * texture coordinate is useful when using proxy texture source.
  *
+ * @param[in] x X coordinate of the picking position.
  * @param[in] y Y coordinate of the picking position.
  * @param[out] node Pointer to receive the node contains the picked mesh.
  * @param[out] mesh Pointer to receive the picked mesh.
@@ -140,26 +145,28 @@ EOAPI void evas_canvas3d_scene_background_color_get(Evas_Real *r, Evas_Real *g,
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Eina_Bool evas_canvas3d_scene_pick(Evas_Real x, Evas_Real y, Evas_Canvas3D_Node **node, Evas_Canvas3D_Mesh **mesh, Evas_Real *s, Evas_Real *t);
+EOAPI Eina_Bool evas_canvas3d_scene_pick(const Eo *obj, Evas_Real x, Evas_Real y, Evas_Canvas3D_Node **node, Evas_Canvas3D_Mesh **mesh, Evas_Real *s, Evas_Real *t);
 
 /**
  * @brief Search given node in the given position.
  *
+ * @param[in] x X coordinate of the picking position.
  * @param[in] y Y coordinate of the picking position.
  * @param[in] node Node for search.
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_exist(Evas_Real x, Evas_Real y, Evas_Canvas3D_Node *node);
+EOAPI Evas_Canvas3D_Node *evas_canvas3d_scene_exist(const Eo *obj, Evas_Real x, Evas_Real y, Evas_Canvas3D_Node *node);
 
 /**
  * @brief Get list of the all root members from scene in the given position.
  *
+ * @param[in] x X coordinate of the picking position.
  * @param[in] y Y coordinate of the picking position.
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Eina_List *evas_canvas3d_scene_pick_member_list_get(Evas_Real x, Evas_Real y);
+EOAPI Eina_List *evas_canvas3d_scene_pick_member_list_get(const Eo *obj, Evas_Real x, Evas_Real y);
 
 /**
  * @brief Enable or disable shadows on given scene
@@ -172,7 +179,7 @@ EOAPI Eina_List *evas_canvas3d_scene_pick_member_list_get(Evas_Real x, Evas_Real
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI void evas_canvas3d_scene_shadows_enable_set(Eina_Bool shadows_enabled);
+EOAPI void evas_canvas3d_scene_shadows_enable_set(Eo *obj, Eina_Bool shadows_enabled);
 
 /**
  * @brief Get shadows enabled status for given scene.
@@ -181,13 +188,13 @@ EOAPI void evas_canvas3d_scene_shadows_enable_set(Eina_Bool shadows_enabled);
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Eina_Bool evas_canvas3d_scene_shadows_enable_get(void);
+EOAPI Eina_Bool evas_canvas3d_scene_shadows_enable_get(const Eo *obj);
 
 /** Get status of color picking of the scene.
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Eina_Bool evas_canvas3d_scene_color_pick_enable_get(void);
+EOAPI Eina_Bool evas_canvas3d_scene_color_pick_enable_get(const Eo *obj);
 
 /**
  * @brief Set posibility color picking.
@@ -196,7 +203,7 @@ EOAPI Eina_Bool evas_canvas3d_scene_color_pick_enable_get(void);
  *
  * @ingroup Evas_Canvas3D_Scene
  */
-EOAPI Eina_Bool evas_canvas3d_scene_color_pick_enable_set(Eina_Bool color_pick);
+EOAPI Eina_Bool evas_canvas3d_scene_color_pick_enable_set(Eo *obj, Eina_Bool color_pick);
 
 
 #endif
