@@ -15,7 +15,7 @@ typedef Eo Evas_VG;
 #endif
 #define EVAS_VG_CLASS evas_vg_class_get()
 
-EAPI const Eo_Class *evas_vg_class_get(void) EINA_CONST;
+EWAPI const Eo_Class *evas_vg_class_get(void);
 
 /**
  * @brief Get the root node of the evas_object_vg.
@@ -29,7 +29,7 @@ EAPI const Eo_Class *evas_vg_class_get(void) EINA_CONST;
  *
  * @ingroup Evas_VG
  */
-EOAPI Efl_VG * evas_obj_vg_root_node_get(void);
+EOAPI Eo_Base *evas_obj_vg_root_node_get(const Eo *obj);
 
 
 #endif
